@@ -32,7 +32,9 @@
 r"""Equality Set Projection (ESP).
 
 Non-vertex polytope projection method from
-    http://www-control.eng.cam.ac.uk/~cnj22/research/projection.html
+- https://web.archive.org/web/20150103142532/
+    https://www-control.eng.cam.ac.uk/~cnj22/research/projection.html
+- https://infoscience.epfl.ch/record/169768
 
 Very unstable, can not handle complex polytopes.
 
@@ -43,9 +45,6 @@ Reference
 \cite{Jones04}
 """
 # Created by P. Nilsson, 8/2/11
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
 import pickle
 
 import numpy as np
@@ -106,7 +105,7 @@ def esp(CC, DD, bb, centered=False, abs_tol=1e-10, verbose=0):
     P = {[C D]x <= b} where C is M x D and D is M x K is
     defined as proj(P) = {x in R^d | exist y in R^k s.t Cx + Dy < b}
     """
-    if 'glpk' in solvers.installed_solvers:
+    if 'glpk' not in solvers.installed_solvers:
         raise Exception(
             "projection_esp error:"
             " Equality set projection requires `cvxopt.glpk` to run.")
@@ -474,12 +473,13 @@ def adjacent(C, D, b, rid_fac, abs_tol=1e-7):
         data["D"] = D
         data["b"] = b
         sio.savemat("matlabdata", data)
-        pickle.dump(data, open("polytope.p", "wb"))
+        with open('polytope.p', 'wb') as f:
+            pickle.dump(data, f)
         raise Exception(
             "adjacent: Lp returned status " + str(sol['status']))
     opt_sol = np.array(sol['x']).flatten()
     dual_opt_sol = np.array(sol['z']).flatten()
-    x_opt = opt_sol[range(0, d)]
+    x_opt = opt_sol[range(d)]
     y_opt = opt_sol[range(d, d + k)]
     if is_dual_degenerate(
             c.flatten(), G, h, A, bf * (1 - 0.01),
@@ -685,7 +685,7 @@ def unique_equalityset2(C, D, b, opt_sol, abs_tol=1e-7):
             "unique_equalityset: LP returned status " +
             str(sol['status']))
     opt_sol2 = np.array(sol['x']).flatten()
-    x = opt_sol2[range(0, n)]
+    x = opt_sol2[range(n)]
     s = opt_sol2[range(n, len(opt_sol2))]
     E = np.nonzero(s > abs_tol)[0]
     print(E)
@@ -717,7 +717,7 @@ def cheby_center(C, D, b):
     sol = solvers.lpsolve(c, G, h=b, solver='glpk')
     if sol['status'] == "optimal":
         opt = np.array(sol['x'][0:-1]).flatten()
-        return opt[range(0, d)], opt[range(d, d + k)], True
+        return opt[range(d)], opt[range(d, d + k)], True
     else:
         return np.zeros(d), np.zeros(k), False
 
@@ -747,8 +747,8 @@ def normalize(AA, bb, abs_tol=1e-7):
     for i in range(len(anorm)):
         unique = True
         for j in range(i + 1, len(anorm)):
-            test = np.sum(np.abs(A[i, :] - A[j, :])) + \
-                np.abs(b[i, 0] - b[j, 0])
+            test = (np.sum(np.abs(A[i, :] - A[j, :])) +
+                np.abs(b[i, 0] - b[j, 0]))
             if test < abs_tol:
                 unique = False
                 break
