@@ -40,7 +40,8 @@
 #
 #  M. Kvasnica, P. Grieder and M. Baotić,
 #  Multi-Parametric Toolbox (MPT),
-#  http://control.ee.ethz.ch/~mpt/
+#  https://people.ee.ethz.ch/~mpt/2/  (Multi-Parametric Toolbox version 2.*)
+#  https://web.archive.org/web/20121011103905/http://control.ee.ethz.ch/~mpt/
 #
 #  mldivide
 #  region_diff
@@ -59,9 +60,7 @@ To change this choice, see the module `polytope.solvers`.
 
 The structure of this module is based on \cite{MPT04}.
 """
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
+import math
 import logging
 import warnings
 
@@ -73,10 +72,7 @@ from polytope.quickhull import quickhull
 
 
 logger = logging.getLogger(__name__)
-try:
-    xrange
-except NameError:
-    xrange = range
+
 
 # Nicer numpy output
 np.set_printoptions(precision=5, suppress=True)
@@ -93,8 +89,13 @@ ABS_TOL = 1e-7
 
 
 class Polytope(object):
-    """Polytope class with following fields
+    """A convex polytope, in half-space representation.
 
+    The minimal vertex representation can be computed with
+    the function `extreme`. A minimal half-space representation can
+    be computed from a vertex representation with the function `qhull`.
+
+    Attributes:
       - `A`: a numpy array for the hyperplane normals in hyperplane
              representation of a polytope
       - `b`: a numpy array for the hyperplane offsets in hyperplane
@@ -109,9 +110,13 @@ class Polytope(object):
       - `dim`: dimension
       - `volume`: volume, computed on first call
 
+    Reference
+    =========
+    https://en.wikipedia.org/wiki/Convex_polytope
+
     See Also
     ========
-    L{Region}
+    L{Region}, L{extreme}, L{qhull}
     """
 
     def __init__(
@@ -128,7 +133,7 @@ class Polytope(object):
             self.b = self.b[pos]
             Anorm = Anorm[pos]
             mult = 1 / Anorm
-            for i in xrange(self.A.shape[0]):
+            for i in range(self.A.shape[0]):
                 self.A[i, :] = self.A[i, :] * mult[i]
             self.b = self.b.flatten() * mult
         self.minrep = minrep
@@ -136,7 +141,10 @@ class Polytope(object):
         self._chebR = chebR
         self.bbox = None
         self.fulldim = fulldim
-        self._volume = volume
+        if volume is not None:
+            self._set_volume(volume)
+        else:
+            self._volume = None
         self.vertices = vertices
 
     def __str__(self):
@@ -251,7 +259,7 @@ class Polytope(object):
         @rtype: L{Polytope} or L{Region}
         """
         if isinstance(other, Region):
-            return other.intersect(self)
+            return other.intersect(self, abs_tol=abs_tol)
         if not isinstance(other, Polytope):
             msg = 'Polytope intersection defined only'
             msg += ' with other Polytope. Got instead: '
@@ -376,6 +384,17 @@ class Polytope(object):
             self._volume = volume(self)
         return self._volume
 
+    def _set_volume(self, polytope_volume):
+        """Set the attribute `self._volume`.
+
+        @param polytope_volume: nonnegative number
+        """
+        if polytope_volume < 0.0:
+            raise ValueError(
+                '`polytope_volume` must be >= 0, given:  {v}'.format(
+                    v=polytope_volume))
+        self._volume = float(polytope_volume)
+
     @property
     def chebR(self):
         r, xc = cheby_ball(self)
@@ -402,13 +421,13 @@ class Polytope(object):
 
     def plot(self, ax=None, color=None, hatch=None, alpha=1.0, linestyle=None, linewidth=None, edgecolor=None):
         if self.dim != 2:
-            raise Exception("Cannot plot polytopes of dimension larger than 2")
-        
+            raise Exception("Cannot plot polytopes of dimension other than 2")
+
         # Setting default values for plotting
         linestyle = linestyle or "dashed"
         linewidth = linewidth or 3
         edgecolor = edgecolor or "black"
-        
+
         ax = _newax(ax)
         if not is_fulldim(self):
             logger.error("Cannot plot empty polytope")
@@ -496,25 +515,37 @@ def _rotate(polyreg, i=None, j=None, u=None, v=None, theta=None, R=None):
     # determine the rotation matrix based on inputs
     if R is not None:
         logger.debug("rotate: R=\n{}".format(R))
-        assert i is None, i
-        assert j is None, j
-        assert theta is None, theta
-        assert u is None, u
-        assert v is None, v
+        if i is not None:
+            raise ValueError(i)
+        if j is not None:
+            raise ValueError(j)
+        if theta is not None:
+            raise ValueError(theta)
+        if u is not None:
+            raise ValueError(u)
+        if v is not None:
+            raise ValueError(v)
     elif i is not None and j is not None and theta is not None:
         logger.info("rotate via indices and angle.")
-        assert R is None, R
-        assert u is None, u
-        assert v is None, v
+        if R is not None:
+            raise ValueError(R)
+        if u is not None:
+            raise ValueError(u)
+        if v is not None:
+            raise ValueError(v)
         if i == j:
             raise ValueError("Must provide two unique basis vectors.")
         R = givens_rotation_matrix(i, j, theta, polyreg.dim)
     elif u is not None and v is not None:
         logger.info("rotate via 2 vectors.")
-        assert R is None, R
-        assert i is None, i
-        assert j is None, j
-        assert theta is None, theta
+        if R is not None:
+            raise ValueError(R)
+        if i is not None:
+            raise ValueError(i)
+        if j is not None:
+            raise ValueError(j)
+        if theta is not None:
+            raise ValueError(theta)
         R = solve_rotation_ap(u, v)
     else:
         raise ValueError("R or (i and j and theta) or (u and v) "
@@ -583,7 +614,7 @@ def solve_rotation_ap(u, v):
         M[1, 1] = -1
         uv = M.dot(uv)
     # align uv plane with the basis01 plane and u with basis0.
-    for c in range(0, 2):
+    for c in range(2):
         for r in range(N - 1, c, -1):
             if uv[r, c] != 0:  # skip rotations when theta will be zero
                 theta = np.arctan2(uv[r, c], uv[r - 1, c])
@@ -613,10 +644,12 @@ def _hessian_normal(A, b):
 
 
 class Region(object):
-    """Class for lists of convex polytopes
+    """A polytope, possibly nonconvex.
 
-    Contains the following fields:
+    Represented using a `list` of convex polytopes.
+    Is usable as a container of polytopes.
 
+    Attributes:
       - `list_poly`: list of Polytope objects
       - `props`: set of propositions inside region
       - `bbox`: if calculated, bounding box of region (see bounding_box)
@@ -627,6 +660,10 @@ class Region(object):
       - `chebXc`: coordinates of maximum chebyshev center (if calculated)
       - `chebR`: maximum chebyshev radius (if calculated)
 
+    Reference
+    =========
+    https://en.wikipedia.org/wiki/Polytope
+
     See Also
     ========
     L{Polytope}
@@ -668,7 +705,7 @@ class Region(object):
 
     def __str__(self):
         output = ''
-        for i in xrange(len(self.list_poly)):
+        for i in range(len(self.list_poly)):
             output += '\t Polytope number ' + str(i + 1) + ':\n'
             poly_str = str(self.list_poly[i])
             poly_str = poly_str.replace('\n', '\n\t\t')
@@ -695,7 +732,8 @@ class Region(object):
         """
         if not isinstance(points, np.ndarray):
             points = np.array(points)
-        assert points.shape[0] == self.dim, 'points should be column vectors'
+        if points.shape[0] != self.dim:
+            raise ValueError('points should be column vectors')
         contained = np.full(points.shape[1], False, dtype=bool)
         for poly in self.list_poly:
             contained = np.logical_or(
@@ -837,6 +875,17 @@ class Region(object):
             self._volume = volume(self)
         return self._volume
 
+    def _set_volume(self, region_volume):
+        """Set the attribute `self._volume`.
+
+        @param region_volume: nonnegative number
+        """
+        if region_volume < 0.0:
+            raise ValueError(
+                '`region_volume` must be >= 0, given:  {v}'.format(
+                    v=region_volume))
+        self._volume = float(region_volume)
+
     @property
     def chebR(self):
         r, xc = cheby_ball(self)
@@ -865,7 +914,7 @@ class Region(object):
         """Plot a `polytope` on axes `ax`."""
         # TODO optional arg for text label
         if self.dim != 2:
-            raise Exception("Cannot plot region of dimension larger than 2")
+            raise Exception("Cannot plot region of dimension other than 2")
         if not is_fulldim(self):
             logger.error("Cannot plot empty region")
             return None
@@ -898,7 +947,7 @@ def is_empty(polyreg):
             return True
     else:
         N = np.zeros(n, dtype=int)
-        for i in xrange(n):
+        for i in range(n):
             N[i] = is_empty(polyreg.list_poly[i])
         if np.all(N):
             return True
@@ -923,7 +972,7 @@ def is_fulldim(polyreg, abs_tol=ABS_TOL):
         status = rc > abs_tol
     else:
         status = np.zeros(lenP)
-        for ii in xrange(lenP):
+        for ii in range(lenP):
             rc, xc = cheby_ball(polyreg.list_poly[ii])
             status[ii] = rc > abs_tol
         status = np.sum(status)
@@ -979,7 +1028,7 @@ def is_inside(polyreg, point, abs_tol=ABS_TOL):
 
 
 def is_subset(small, big, abs_tol=ABS_TOL):
-    """Return True if small \subseteq big.
+    r"""Return True if small \subseteq big.
 
     @type small: L{Polytope} or L{Region}
     @type big:   L{Polytope} or L{Region}
@@ -1005,7 +1054,7 @@ def reduce(poly, nonEmptyBounded=1, abs_tol=ABS_TOL):
     Uses the algorithm described at [1],
     by solving one LP for each facet.
 
-    [1] http://www.ifor.math.ethz.ch/~fukuda/polyfaq/node24.html
+    [1] https://www.cs.mcgill.ca/~fukuda/soft/polyfaq/node24.html
 
     Warning:
       - nonEmptyBounded == 0 case is not tested much.
@@ -1039,18 +1088,24 @@ def reduce(poly, nonEmptyBounded=1, abs_tol=ABS_TOL):
     neq = np.shape(A_arr)[0]
     # first eliminate the linearly dependent rows
     # corresponding to the same hyperplane
-    M1 = np.hstack([A_arr, np.array([b_arr]).T]).T
-    M1row = 1 / np.sqrt(np.sum(M1**2, 0))
-    M1n = np.dot(M1, np.diag(M1row))
-    M1n = M1n.T
-    keep_row = []
-    for i in xrange(neq):
-        keep_i = 1
-        for j in xrange(i + 1, neq):
-            if np.dot(M1n[i].T, M1n[j]) > 1 - abs_tol:
-                keep_i = 0
-        if keep_i:
-            keep_row.append(i)
+    # Normalize all rows
+    a_norm = 1 / np.sqrt(np.sum(A_arr.T**2, 0))
+    a_normed = np.dot(A_arr.T, np.diag(a_norm)).T
+    remove_row = []
+    for i in range(neq):
+        for j in range(i + 1, neq):
+            # If the product of two vectors are close to 1,
+            # since they are both unit vectors,
+            # they must represent parallel hyperplanes
+            if np.dot(a_normed[i].T, a_normed[j]) > 1 - abs_tol:
+                # Check which inequality that constrains the most
+                b_in = b_arr[i] * a_norm[i]
+                b_jn = b_arr[j] * a_norm[j]
+                if b_in < b_jn:
+                    remove_row.append(j)
+                else:
+                    remove_row.append(i)
+    keep_row = np.setdiff1d(range(neq), remove_row).tolist()
     A_arr = A_arr[keep_row]
     b_arr = b_arr[keep_row]
     neq, nx = A_arr.shape
@@ -1060,8 +1115,17 @@ def reduce(poly, nonEmptyBounded=1, abs_tol=ABS_TOL):
     # Now eliminate hyperplanes outside the bounding box
     if neq > 3 * nx:
         lb, ub = Polytope(A_arr, b_arr).bounding_box
-        # cand = -(np.dot((A_arr>0)*A_arr,ub-lb)
-        #-(b_arr-np.dot(A_arr,lb).T).T<-1e-4)
+        # Do a coordinate system translation such that the lower bound is
+        # moved to the origin
+        #       A*(x-lb) <= b - A*lb
+        # Relative to the origin, a row ai in A with only positive coefficients
+        # represents an upper bound. If ai*(x1-lb) <= bi,
+        # the hyperplane is above x1.
+        # Hence, if ai*(ub-lb) <= bi, then the hyperplane at row i
+        # does not intersect the bounding box.
+        # The same holds for rows with negative coefficients multiplied with
+        # the origin. Rows with both negative and positive coefficients
+        # are a mixture of the two extremes.
         cand = ~ (np.dot((A_arr > 0) * A_arr, ub - lb) -
                   (np.array([b_arr]).T - np.dot(A_arr, lb)) < -1e-4)
         A_arr = A_arr[cand.squeeze()]
@@ -1070,15 +1134,23 @@ def reduce(poly, nonEmptyBounded=1, abs_tol=ABS_TOL):
     if nonEmptyBounded:
         if neq <= nx + 1:
             return Polytope(A_arr, b_arr)
+    # Check for each inequality whether it is implied by
+    # the other inequalities, i.e., is it redundant?
     del keep_row[:]
-    for k in xrange(A_arr.shape[0]):
+    for k in range(neq):
+        # Setup object function to maximize the linear function
+        # defined as current row of A matrix
         f = -A_arr[k, :]
         G = A_arr
         h = b_arr
+        # Give some slack in the current inequality
         h[k] += 0.1
         sol = lpsolve(f, G, h)
         h[k] -= 0.1
         if sol['status'] == 0:
+            # If the maximum is greater than the constraint of
+            # the inequality, then the inequality constrains solutions
+            # and thus the inequality is non-redundant
             obj = -sol['fun'] - h[k]
             if obj > abs_tol:
                 keep_row.append(k)
@@ -1145,7 +1217,7 @@ def union(polyreg1, polyreg2, check_convex=False):
             # Check convexity for each pair of polytopes
             while N > 0:
                 templist = [lst[0]]
-                for ii in xrange(1, N):
+                for ii in range(1, N):
                     templist.append(lst[ii])
                     is_conv, env = is_convex(Region(templist))
                     if not is_conv:
@@ -1178,11 +1250,11 @@ def cheby_ball(poly1):
 
     Example (low dimension):
 
-    r1,x1 = cheby_ball(P, [1]) calculates the center and half the
+    r1,x1 = cheby_ball(P) calculates the center and half the
     length of the longest line segment along the first coordinate axis
     inside polytope P
 
-    @type poly1: L{Polytope}
+    @type poly1: L{Polytope} or L{Region}
 
     @return: rc,xc: Chebyshev radius rc (float) and center xc (numpy array)
     """
@@ -1226,6 +1298,17 @@ def cheby_ball(poly1):
     return poly1._chebR, poly1._chebXc
 
 
+def _bounding_box_to_polytope(lower, upper):
+    """Return a `Polytope` that represents the given bounding box.
+
+    @param lower: corner point of the bounding box
+    @param upper: corner point of the bounding box
+    @rtype: `Polytope`
+    """
+    intervals = [(a[0], b[0]) for a, b in zip(lower, upper)]
+    return box2poly(intervals)
+
+
 def bounding_box(polyreg):
     """Return smallest hyperbox containing polytope or region.
 
@@ -1260,7 +1343,7 @@ def bounding_box(polyreg):
         alllower = np.zeros([lenP, dimP])
         allupper = np.zeros([lenP, dimP])
         # collect lower and upper bounds
-        for ii in xrange(0, lenP):
+        for ii in range(lenP):
             bbox = polyreg.list_poly[ii].bounding_box
             ll, uu = bbox
             alllower[ii, :] = ll.T
@@ -1268,7 +1351,7 @@ def bounding_box(polyreg):
         l = np.zeros([dimP, 1])
         u = np.zeros([dimP, 1])
         # compute endpoints
-        for ii in xrange(0, dimP):
+        for ii in range(dimP):
             l[ii] = min(alllower[:, ii])
             u[ii] = max(allupper[:, ii])
         polyreg.bbox = l, u
@@ -1279,7 +1362,7 @@ def bounding_box(polyreg):
     l = np.zeros([n, 1])
     u = np.zeros([n, 1])
     # lower corner
-    for i in xrange(0, n):
+    for i in range(n):
         c = np.array(In[:, i])
         G = polyreg.A
         h = polyreg.b
@@ -1287,8 +1370,14 @@ def bounding_box(polyreg):
         if sol['status'] == 0:
             x = sol['x']
             l[i] = x[i]
+        else:
+            raise RuntimeError((
+                '`polytope.solvers.lpsolve` returned:  {v}\n'
+                'its docstring describes return values'
+                ).format(
+                    v=sol))
     # upper corner
-    for i in xrange(0, n):
+    for i in range(n):
         c = np.negative(np.array(In[:, i]))
         G = polyreg.A
         h = polyreg.b
@@ -1296,6 +1385,12 @@ def bounding_box(polyreg):
         if sol['status'] == 0:
             x = sol['x']
             u[i] = x[i]
+        else:
+            raise RuntimeError((
+                '`polytope.solvers.lpsolve` returned:  {v}\n'
+                'its docstring describes return values'
+                ).format(
+                    v=sol))
     polyreg.bbox = l, u
     return l, u
 
@@ -1318,14 +1413,14 @@ def envelope(reg, abs_tol=ABS_TOL):
     Ae = None
     be = None
     nP = len(reg.list_poly)
-    for i in xrange(nP):
+    for i in range(nP):
         poly1 = reg.list_poly[i]
         outer_i = np.ones(poly1.A.shape[0])
-        for ii in xrange(poly1.A.shape[0]):
+        for ii in range(poly1.A.shape[0]):
             if outer_i[ii] == 0:
                 # If inequality already discarded
                 continue
-            for j in xrange(nP):
+            for j in range(nP):
                 # Check for each polytope
                 # if it intersects with inequality ii
                 if i == j:
@@ -1346,7 +1441,7 @@ def envelope(reg, abs_tol=ABS_TOL):
         else:
             Ae = np.vstack([Ae, poly1.A[ind_i, :]])
             be = np.hstack([be, poly1.b[ind_i]])
-    ret = reduce(Polytope(Ae, be))
+    ret = reduce(Polytope(Ae, be), abs_tol=abs_tol)
     if is_fulldim(ret):
         return ret
     else:
@@ -1357,7 +1452,7 @@ count = 0
 
 
 def mldivide(a, b, save=False):
-    """Return set difference a \ b.
+    r"""Return set difference a \ b.
 
     @param a: L{Polytope} or L{Region}
     @param b: L{Polytope} to subtract
@@ -1405,9 +1500,9 @@ def intersect(poly1, poly2, abs_tol=ABS_TOL):
     # raise NotImplementedError('Being removed,
     # use {Polytope, Region}.intersect instead')
     if isinstance(poly1, Region):
-        return poly1.intersect(poly2)
+        return poly1.intersect(poly2, abs_tol=abs_tol)
     if isinstance(poly2, Region):
-        return poly2.intersect(poly1)
+        return poly2.intersect(poly1, abs_tol=abs_tol)
     if not isinstance(poly1, Polytope):
         msg = 'poly1 not Region nor Polytope.'
         msg += 'Got instead: ' + str(type(poly1))
@@ -1415,28 +1510,39 @@ def intersect(poly1, poly2, abs_tol=ABS_TOL):
     return poly1.intersect(poly2, abs_tol)
 
 
-def volume(polyreg):
+def volume(polyreg, nsamples=None, seed=None):
     """Approximately compute the volume of a Polytope or Region.
 
     A randomized algorithm is used.
 
     @type polyreg: L{Polytope} or L{Region}
+    @param nsamples: number of samples to generate to
+        use for estimating volume
+    @type nsamples: positive integer
+    @param seed: initialization for the random number
+        generator. Passed as argument to the parameter
+        `seed` of the function `numpy.random.default_rng`,
+        read the docstring of that function for details.
+
+        The seed can be used for reproducible volume
+        computations. The documentation of the class
+        `numpy.random.SeedSequence` includes useful
+        recommendations for how to initialize a random
+        generator automatically, and record the seed
+        for reusing it.
 
     @return: Volume of input
     """
     if not is_fulldim(polyreg):
         return 0.0
-    try:
-        if polyreg._volume is not None:
-            return polyreg._volume
-    except Exception:
-        logger.debug('computing volume...')
+    if polyreg._volume is not None:
+        logger.debug('recomputing polytope volume...')
     # `Region` ?
     if isinstance(polyreg, Region):
-        tot_vol = 0.
-        for i in xrange(len(polyreg)):
+        tot_vol = 0.0
+        for i in range(len(polyreg)):
             tot_vol += volume(polyreg.list_poly[i])
-        polyreg._volume = tot_vol
+        polyreg._set_volume(tot_vol)
         return tot_vol
     # `polyreg` is a `Polytope`
     n = polyreg.A.shape[1]
@@ -1448,15 +1554,27 @@ def volume(polyreg):
         N = 3000
     else:
         N = 10000
+    if nsamples is not None and nsamples < 1:
+        raise ValueError(
+            '`nsamples` must be >= 1, given:  {v}'.format(
+                v=nsamples))
+    if nsamples is not None:
+        N = nsamples
+    if N != int(N):
+        raise ValueError((
+            'it appears that a noninteger number of samples '
+            'has been given, namely:  {v}'
+            ).format(
+                v=nsamples))
     l_b, u_b = polyreg.bounding_box
     x = (np.tile(l_b, (1, N))
-         + np.random.rand(n, N)
+         + np.random.default_rng(seed).random((n, N))
          * np.tile(u_b - l_b, (1, N)))
     aux = (np.dot(polyreg.A, x)
            - np.tile(np.array([polyreg.b]).T, (1, N)))
     aux = np.nonzero(np.all(aux < 0, 0))[0].shape[0]
     vol = np.prod(u_b - l_b) * aux / N
-    polyreg._volume = vol
+    polyreg._set_volume(vol)
     return vol
 
 
@@ -1487,7 +1605,7 @@ def extreme(poly1):
     # distinguish cases by dimension
     if nx == 1:
         # Polytope is a 1-dim line
-        for ii in xrange(nc):
+        for ii in range(nc):
             V = np.append(V, b[ii] / A[ii])
         if len(A) == 1:
             R = np.append(R, 1)
@@ -1499,7 +1617,7 @@ def extreme(poly1):
         H = np.vstack([A, A[0, :]])
         K = np.hstack([b, b[0]])
         I = np.hstack([I, I[0]])
-        for ii in xrange(nc):
+        for ii in range(nc):
             HH = np.vstack([H[I[ii], :], H[I[ii + 1], :]])
             KK = np.hstack([K[I[ii]], K[I[ii + 1]]])
             if np.linalg.cond(HH) == np.inf:
@@ -1526,7 +1644,7 @@ def extreme(poly1):
         b = poly1.b.copy()
         sh = np.shape(A)
         Ai = np.zeros(sh)
-        for ii in xrange(sh[0]):
+        for ii in range(sh[0]):
             Ai[ii, :] = A[ii, :] / (b[ii] - np.dot(A[ii, :], xmid))
         Q = reduce(qhull(Ai))
         if not is_fulldim(Q):
@@ -1537,11 +1655,12 @@ def extreme(poly1):
         sh = np.shape(H)
         nx = sh[1]
         V = np.zeros(sh)
-        for iv in xrange(sh[0]):
-            for ix in xrange(nx):
+        for iv in range(sh[0]):
+            for ix in range(nx):
                 V[iv, ix] = H[iv, ix] / K[iv] + xmid[ix]
     a = V.size / nx
-    assert a.is_integer(), a
+    if not a.is_integer():
+        raise AssertionError(a)
     a = int(a)
     poly1.vertices = V.reshape((a, nx))
     return poly1.vertices
@@ -1586,7 +1705,7 @@ def projection(poly1, dim, solver=None, abs_tol=ABS_TOL, verbose=0):
     """
     if isinstance(poly1, Region):
         ret = Polytope()
-        for i in xrange(len(poly1.list_poly)):
+        for i in range(len(poly1.list_poly)):
             p = projection(
                 poly1.list_poly[i], dim,
                 solver=solver, abs_tol=abs_tol)
@@ -1598,7 +1717,7 @@ def projection(poly1, dim, solver=None, abs_tol=ABS_TOL, verbose=0):
     # `poly1` isn't flat
     poly_dim = poly1.dim
     dim = np.array(dim)
-    org_dim = xrange(poly_dim)
+    org_dim = range(poly_dim)
     new_dim = dim.flatten() - 1
     del_dim = np.setdiff1d(org_dim, new_dim)  # Index of dimensions to remove
     # logging
@@ -1667,7 +1786,7 @@ def separate(reg1, abs_tol=ABS_TOL):
     @return: List [] of connected Regions
     """
     final = []
-    ind_left = xrange(len(reg1))
+    ind_left = range(len(reg1))
     props = reg1.props
     while len(ind_left) > 0:
         ind_del = []
@@ -1675,7 +1794,7 @@ def separate(reg1, abs_tol=ABS_TOL):
             [reg1.list_poly[ind_left[0]]],
             [])
         ind_del.append(ind_left[0])
-        for i in xrange(1, len(ind_left)):
+        for i in range(1, len(ind_left)):
             j = ind_left[i]
             if is_adjacent(connected_reg, reg1.list_poly[j]):
                 connected_reg = union(
@@ -1788,7 +1907,7 @@ def projection_fm(poly1, new_dim, del_dim, abs_tol=ABS_TOL):
     poly = poly1.copy()
     for i in del_dim:
         positive = np.nonzero(poly.A[:, i] > abs_tol)[0]
-        negative = np.nonzero(poly.A[:, i] < abs_tol)[0]
+        negative = np.nonzero(poly.A[:, i] < -abs_tol)[0]
         null = np.nonzero(np.abs(poly.A[:, i]) < abs_tol)[0]
         nr = len(null) + len(positive) * len(negative)
         nc = np.shape(poly.A)[0]
@@ -1843,11 +1962,23 @@ def projection_iterhull(poly1, new_dim, max_iter=1000,
         sol = lpsolve(f1, poly1.A, poly1.b)
         if sol['status'] == 0:
             vert1 = sol['x']
+        else:
+            raise RuntimeError((
+                '`polytope.solvers.lpsolve` returned:  {v}\n'
+                'its docstring describes return values'
+                ).format(
+                    v=sol))
         sol = lpsolve(np.negative(f1), poly1.A, poly1.b)
         if sol['status'] == 0:
             vert2 = sol['x']
+        else:
+            raise RuntimeError((
+                '`polytope.solvers.lpsolve` returned:  {v}\n'
+                'its docstring describes return values'
+                ).format(
+                    v=sol))
         vert = np.vstack([vert1, vert2])
-        return qhull(vert)
+        return qhull(vert, abs_tol=abs_tol)
     else:
         OK = False
         cnt = 0
@@ -1885,7 +2016,7 @@ def projection_iterhull(poly1, new_dim, max_iter=1000,
         logger.debug("Found starting simplex after " +
                      str(cnt) + " iterations")
         cnt = 0
-        P1 = qhull(Vert[:, new_dim])
+        P1 = qhull(Vert[:, new_dim], abs_tol=abs_tol)
         HP = None
         while True:
             # Iteration:
@@ -1896,7 +2027,7 @@ def projection_iterhull(poly1, new_dim, max_iter=1000,
                 raise Exception("iterative_hull: "
                                 "maximum number of iterations reached")
             logger.debug("Iteration number " + str(cnt))
-            for ind in xrange(P1.A.shape[0]):
+            for ind in range(P1.A.shape[0]):
                 f1 = np.round(P1.A[ind, :] / abs_tol) * abs_tol
                 f2 = np.hstack([np.round(P1.A[ind, :] / abs_tol) * abs_tol,
                                 np.round(P1.b[ind] / abs_tol) * abs_tol])
@@ -1904,7 +2035,7 @@ def projection_iterhull(poly1, new_dim, max_iter=1000,
                 k = np.array([])
                 if HP is not None:
                     k = np.nonzero(HP[:, 0] == f2[0])[0]
-                    for j in xrange(1, np.shape(P1.A)[1] + 1):
+                    for j in range(1, np.shape(P1.A)[1] + 1):
                         ii = np.nonzero(HP[k, j] == f2[j])[0]
                         k = k[ii]
                         if k.size == 0:
@@ -1935,12 +2066,12 @@ def projection_iterhull(poly1, new_dim, max_iter=1000,
                         HP = np.vstack([HP, add])
                     Vert = np.vstack([Vert, xopt])
             logger.debug("Taking convex hull of new points")
-            P2 = qhull(Vert[:, new_dim])
+            P2 = qhull(Vert[:, new_dim], abs_tol=abs_tol)
             logger.debug("Checking if new points are inside convex hull")
             OK = 1
-            for i in xrange(np.shape(Vert)[0]):
+            for i in range(np.shape(Vert)[0]):
                 if not P1.contains(np.transpose([Vert[i, new_dim]]),
-                                   abs_tol=1e-5):
+                                   abs_tol=abs_tol):
                     # If all new points are inside
                     # old polytope -> Finished
                     OK = 0
@@ -2019,13 +2150,13 @@ def region_diff(poly, reg, abs_tol=ABS_TOL, intersect_tol=ABS_TOL,
     mi = np.zeros(N, dtype=int)
     # Finding constraints that are not in original polytope
     HK = np.hstack([H, np.array([K]).T])
-    for ii in xrange(N):
+    for ii in range(N):
         i = ind[ii]
         if not is_fulldim(reg.list_poly[i]):
             continue
         Hni = reg.list_poly[i].A.copy()
         Kni = reg.list_poly[i].b.copy()
-        for j in xrange(np.shape(Hni)[0]):
+        for j in range(np.shape(Hni)[0]):
             HKnij = np.hstack([Hni[j, :], Kni[j]])
             HK2 = np.tile(HKnij, [m, 1])
             abs = np.abs(HK - HK2)
@@ -2062,7 +2193,7 @@ def region_diff(poly, reg, abs_tol=ABS_TOL, intersect_tol=ABS_TOL,
             if save:
                 logger.debug('counter[level] is 0')
 
-            for j in xrange(level, N):
+            for j in range(level, N):
                 auxINDICES = np.hstack([
                     INDICES,
                     range(beg_mi[j], beg_mi[j] + mi[j])
@@ -2079,7 +2210,7 @@ def region_diff(poly, reg, abs_tol=ABS_TOL, intersect_tol=ABS_TOL,
                 level = level - 1
                 res = union(res, Polytope(A[INDICES, :], B[INDICES]), False)
                 nzcount = np.nonzero(counter)[0]
-                for jj in xrange(len(nzcount) - 1, -1, -1):
+                for jj in range(len(nzcount) - 1, -1, -1):
                     if counter[level] <= mi[level]:
                         INDICES[len(INDICES) -
                                 1] = INDICES[len(INDICES) - 1] - M
@@ -2099,7 +2230,7 @@ def region_diff(poly, reg, abs_tol=ABS_TOL, intersect_tol=ABS_TOL,
                 logger.debug('counter[level] > 0')
             # counter(level) > 0
             nzcount = np.nonzero(counter)[0]
-            for jj in xrange(len(nzcount) - 1, -1, -1):
+            for jj in range(len(nzcount) - 1, -1, -1):
                 level = nzcount[jj]
                 counter[level] = counter[level] + 1
                 if counter[level] <= mi[level]:
@@ -2140,7 +2271,7 @@ def num_bin(N, places=8):
 
     E.g., given N=7, num_bin returns [1, 1, 1, 0, 0, 0, 0, 0].
     """
-    return [(N >> k) & 0x1 for k in xrange(places)]
+    return [(N >> k) & 0x1 for k in range(places)]
 
 
 def box2poly(box):
@@ -2190,31 +2321,61 @@ def _get_patch(poly1, **kwargs):
     return patch
 
 
+def enumerate_integral_points(poly):
+    """Return all points in `poly` that have integer coordinates.
+
+    @param poly: polytope
+    @type poly: `polytope.polytope.Region` or
+        `polytope.polytope.Polytope`
+    @return: coordinates of `m` points as `d X m` array,
+        where `d` the dimension
+    @rtype: `numpy.ndarray`
+    """
+    a, b = poly.bounding_box
+    a_int = np.floor(a)
+    b_int = np.ceil(b)
+    intervals = list(zip(a_int.flatten(), b_int.flatten()))
+    box = box2poly(intervals)
+    res = [int(b - a + 1) for a, b in intervals]
+    grid, _ = grid_region(box, res=res)
+    inside = poly.contains(grid)
+    return grid[:, inside]
+
+
 def grid_region(polyreg, res=None):
     """Return bounding box grid points within `polyreg`.
 
     @type polyreg: L{Polytope} or L{Region}
-    @param res: resolution of grid
+    @param res: resolution of grid, i.e., how many points
+        the grid has per dimension, before filtering based
+        on which points are contained in the polytope
+    @type res: `list` of `int` >= 1,
+        with `len` equal to `polyreg.dim`
     """
     # grid corners
     bbox = polyreg.bounding_box
-    bbox = np.hstack(bbox)
-    dom = bbox.flatten()
     # grid resolution
-    density = 8
     if res is None:
-        res = list()
-        for i in xrange(0, dom.size, 2):
-            L = dom[i + 1] - dom[i]
-            res += [density * L]
+        density = 8
+        res = [
+            math.ceil(density * (b - a))
+            for a, b in zip(*bbox)]
+    if len(res) != polyreg.dim:
+        raise ValueError((
+            "`len(res)` must equal the polytope's dimension "
+            "(which is {dim}), but instead `res` is:  {res}"
+            ).format(dim=polyreg.dim, res=res))
+    if any(n < 1 for n in res):
+        raise ValueError((
+            '`res` must contain `int` values >= 1, '
+            'instead `res` equals:  {res}'
+            ).format(res=res))
     linspaces = list()
-    for i, n in enumerate(res):
-        a = dom[2 * i]
-        b = dom[2 * i + 1]
-        r = np.linspace(a, b, n)
+    for a, b, n in zip(*bbox, res):
+        r = np.linspace(a, b, num=n)
         linspaces.append(r)
     points = np.meshgrid(*linspaces)
-    x = np.vstack(map(np.ravel, points))
+    x = np.vstack(list(map(np.ravel, points)))
     x = x[:, polyreg.contains(x)]
     return (x, res)
 
@@ -2241,6 +2402,11 @@ def simplices2polytopes(points, triangles):
 
     @type points: N x d
     @type triangles: NT x 3
+
+    References
+    ==========
+    https://en.wikipedia.org/wiki/Simplex
+    https://en.wikipedia.org/wiki/Triangle_mesh
     """
     polytopes = []
     for triangle in triangles:
